@@ -24,7 +24,7 @@ import cv2
 import numpy as np
 import time
 
-import tools.infer.utility as utility
+import pyxlpr.ppocr.tools.infer.utility as utility
 from pyxlpr.ppocr.data import create_operators, transform
 from pyxlpr.ppocr.postprocess import build_post_process
 from pyxlpr.ppocr.utils.logging import get_logger
